package com.ningdatech.carapi.report.controller;

import javax.validation.Valid;

import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.carapi.report.manage.MortarSupplyBulletinManage;
import com.ningdatech.carapi.report.model.po.ReqDataUploadListPO;
import com.ningdatech.carapi.report.model.po.ReqMortarSupplyDataUploadPO;
import com.ningdatech.carapi.report.model.vo.ResDataUploadListVO;
import com.ningdatech.carapi.report.model.vo.ResMortarSupplyDataUploadDetailVO;
import com.ningdatech.carapi.report.model.vo.ResMortarSupplyRelatedDataVO;
import com.ningdatech.log.annotation.WebLog;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

/**
 * <p>
 * 散装水泥直报系统 前端控制器
 * </p>
 *
 * @author CMM
 * @since 2023-09-27
 */
@RequestMapping("/api/v1/direct/report/system")
@RestController
@Api(tags = "数据上报-砂浆供应量快报")
@Validated
@RequiredArgsConstructor
public class MortarSupplyBulletinController {

    private final MortarSupplyBulletinManage mortarSupplyBulletinManage;

    @PostMapping("/get/mortar-supply/related/data")
    @ApiOperation("获取砂浆供应量相关数据")
    public ResMortarSupplyRelatedDataVO getMortarSupplyRelatedData(@Valid @RequestBody ReqMortarSupplyDataUploadPO po) {
        return mortarSupplyBulletinManage.getMortarSupplyRelatedData(po);
    }
    @PostMapping("/get/mortar-supply/upload/data")
    @ApiOperation("获取砂浆供应量快报计算数据")
    public ResMortarSupplyDataUploadDetailVO getMortarSupplyUploadData(@Valid @RequestBody ReqMortarSupplyDataUploadPO po) {
        return mortarSupplyBulletinManage.getMortarSupplyUploadData(po);
    }
    @PostMapping("/mortar-supply/data/upload")
    @ApiOperation("砂浆供应量快报数据上报")
    @WebLog(value = "砂浆供应量快报数据上报", modular = "数据上报-砂浆供应量快报数据上报")
    public String mortarSupplyDataUpload(@Valid @RequestBody ReqMortarSupplyDataUploadPO po) {
        return mortarSupplyBulletinManage.mortarSupplyDataUpload(po);
    }
    @PostMapping("/mortar-supply/data/upload/list")
    @ApiOperation("散装水泥供应量数据上报列表")
    public PageVo<ResDataUploadListVO> dataUploadList(@Valid @RequestBody ReqDataUploadListPO po) {
        return mortarSupplyBulletinManage.dataUploadList(po);
    }
}
