package com.ningdatech.carapi.report.controller;

import javax.validation.Valid;

import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.carapi.report.manage.NonProhibitedAreasBulkCementSalesUseManage;
import com.ningdatech.carapi.report.model.po.ReqDataUploadListPO;
import com.ningdatech.carapi.report.model.po.ReqNonProhibitedAreasDataUploadPO;
import com.ningdatech.carapi.report.model.vo.ResDataUploadListVO;
import com.ningdatech.carapi.report.model.vo.ResNonProhibitedAreasDataUploadDetailVO;
import com.ningdatech.log.annotation.WebLog;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

/**
 * <p>
 * 散装水泥直报系统 前端控制器
 * </p>
 *
 * @author CMM
 * @since 2023-09-27
 */
@RequestMapping("/api/v1/direct/report/system")
@RestController
@Api(tags = "数据上报-非禁现区域散装水泥销售使用情况")
@Validated
@RequiredArgsConstructor
public class NonProhibitedAreasBulkCementSalesUseController {

    private final NonProhibitedAreasBulkCementSalesUseManage nonProhibitedAreasBulkCementSalesUseManage;

    @PostMapping("/get/non-prohibited-areas/upload/data")
    @ApiOperation("获取非禁现区散装水泥销售使用情况年报计算数据")
    public ResNonProhibitedAreasDataUploadDetailVO getBulkCementSupplyUploadData(@Valid @RequestBody ReqNonProhibitedAreasDataUploadPO po) {
        return nonProhibitedAreasBulkCementSalesUseManage.getNonProhibitedAreasUploadData(po);
    }
    @PostMapping("/non-prohibited-areas/data/upload")
    @ApiOperation("非禁现区散装水泥销售使用情况年报数据上报")
    @WebLog(value = "非禁现区散装水泥销售使用情况年报数据上报", modular = "数据上报-非禁现区散装水泥销售使用情况年报数据上报")
    public String bulkCementSupplyDataUpload(@Valid @RequestBody ReqNonProhibitedAreasDataUploadPO po) {
        return nonProhibitedAreasBulkCementSalesUseManage.nonProhibitedAreasDataUpload(po);
    }
    @PostMapping("/non-prohibited-areas/data/upload/list")
    @ApiOperation("非禁现区散装水泥销售使用情况年报数据上报列表")
    public PageVo<ResDataUploadListVO> dataUploadList(@Valid @RequestBody ReqDataUploadListPO po) {
        return nonProhibitedAreasBulkCementSalesUseManage.dataUploadList(po);
    }
}
