package com.ningdatech.carapi.sys.entity.dto;

import com.baomidou.mybatisplus.annotation.TableField;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

import static com.baomidou.mybatisplus.annotation.SqlCondition.LIKE;

/**
 * <p>
 * 实体类
 * 用户
 * </p>
 *
 * @author PoffyZhang
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Accessors(chain = true)
@ToString(callSuper = true)
@EqualsAndHashCode(callSuper = false)
@Builder
@ApiModel(value = "UserSaveDTO", description = "用户")
public class UserSaveDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 账号
     */
    @ApiModelProperty(value = "账号")
    @NotEmpty(message = "请输入账号!")
    @Size(max = 50, message = "账号长度不能超过30")
    private String account;

    /**
     * 用户姓名
     */
    @ApiModelProperty(value = "用户姓名")
    @NotEmpty(message = "请输入用户姓名!")
    @Size(max = 50, message = "用户姓名长度不能超过50")
    private String username;

    /**
     * 角色
     */
    @ApiModelProperty(value = "角色")
    @NotNull(message = "请选择角色")
    private Long role;

    /**
     * 企业
     */
    @ApiModelProperty(value = "企业id")
    @NotNull(message = "请选择企业")
    private Long companyId;

    /**
     * 公司行业
     */
    @ApiModelProperty(value = "行业id")
    private Long industry;

    /**
     * 手机
     */
    @ApiModelProperty(value = "手机")
    @NotBlank(message = "手机号不能为空")
    @Size(max = 20, message = "手机长度不能超过20")
    private String mobile;

    /**
     * 密码
     */
    @ApiModelProperty(value = "密码")
    @NotEmpty(message = "密码不能为空")
    @Size(max = 50, message = "密码长度不能超过64")
    private String password;

    /**
     * 职位
     */
    @ApiModelProperty(value = "职位")
    @Size(max = 50, message = "职位长度不能超过50")
    private String position;

    /**
     * 科室
     */
    @ApiModelProperty(value = "科室")
    @Size(max = 50, message = "科室长度不能超过50")
    private String sectionOffice;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    @Size(max = 200, message = "备注不能超过200")
    private String remark;
}
